llvmbot wrote:

<!--LLVM PR SUMMARY COMMENT-->

@llvm/pr-subscribers-clang

<details>
<summary>Changes</summary>

D63960 performs a tree transformation on AST to prevent evaluating constant 
expressions eagerly by removing recorded immediate consteval invocations from 
subexpressions. (See https://reviews.llvm.org/D63960#inline-600736 for its 
motivation.)

However, the UDL node has been replaced with a CallExpr in the default 
TreeTransform since ca844ab0 (inadvertently?). This confuses clangd as it 
relies on the exact AST node type to decide whether or not to present inlay 
hints for an expression.

With regard to the fix, I think it's enough to return the UDL expression as-is 
during the transformation: We've bound it to temporary in its construction, and 
there's no ConstantExpr to visit under a UDL.

Fixes https://github.com/llvm/llvm-project/issues/63898.
---
Full diff: https://github.com/llvm/llvm-project/pull/66641.diff


2 Files Affected:

- (modified) clang/lib/Sema/SemaExpr.cpp (+4-1) 
- (added) clang/test/AST/ast-dump-udl-consteval.cpp (+17) 


``````````diff
diff --git a/clang/test/AST/ast-dump-udl-consteval.cpp 
b/clang/test/AST/ast-dump-udl-consteval.cpp
new file mode 100644
index 000000000000000..9da53f725172aba
--- /dev/null
+++ b/clang/test/AST/ast-dump-udl-consteval.cpp
@@ -0,0 +1,17 @@
+// RUN: %clang_cc1 -xc++ -std=c++23 -ast-dump %s | FileCheck %s
+
+int inline consteval operator""_u32(unsigned long long val) {
+  return val;
+}
+
+void udl() {
+  (void)(0_u32 + 1_u32);
+}
+
+// CHECK: `-BinaryOperator {{.+}} <col:10, col:18> 'int' '+'
+// CHECK-NEXT: |-ConstantExpr {{.+}} <col:10> 'int'
+// CHECK-NEXT: | |-value: Int 0
+// CHECK-NEXT: | `-UserDefinedLiteral {{.+}} <col:10> 'int'
+// CHECK: `-ConstantExpr {{.+}} <col:18> 'int'
+// CHECK-NEXT:   |-value: Int 1
+// CHECK-NEXT:   `-UserDefinedLiteral {{.+}} <col:18> 'int'

``````````

</details>


https://github.com/llvm/llvm-project/pull/66641
_______________________________________________
cfe-commits mailing list
cfe-commits@lists.llvm.org
https://lists.llvm.org/cgi-bin/mailman/listinfo/cfe-commits

Reply via email to
diff --git a/clang/lib/Sema/SemaExpr.cpp b/clang/lib/Sema/SemaExpr.cpp
index 92496b03ecabe54..104da822aae2930 100644
--- a/clang/lib/Sema/SemaExpr.cpp
+++ b/clang/lib/Sema/SemaExpr.cpp
@@ -18438,7 +18438,10 @@ static void RemoveNestedImmediateInvocation(
       DRSet.erase(cast<DeclRefExpr>(E->getCallee()->IgnoreImplicit()));
       return Base::TransformCXXOperatorCallExpr(E);
     }
-    /// Base::TransformInitializer skip ConstantExpr so we need to visit them
+    /// Base::TransformUserDefinedLiteral doesn't preserve the
+    /// UserDefinedLiteral node.
+    ExprResult TransformUserDefinedLiteral(UserDefinedLiteral *E) { return E; }
+    /// Base::TransformInitializer skips ConstantExpr so we need to visit them
     /// here.
     ExprResult TransformInitializer(Expr *Init, bool NotCopyInit) {
       if (!Init)